package chat.server;

import static chat.Globals.*;
import static chat.Cmds.*;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;

import inout.In;
import inout.Out;

public class ChatServer {
	
	private ServerSocket serverSocket; 
	
	private volatile boolean terminate = false; 
	
	public static void main(String[] args) {
		ChatServer server = new ChatServer(); 
		server.start(); 
	}

	private void start() {
		Out.println("Started chat server");
		new Thread(() -> {
			Socket clientSocket = null; 
			try {
				serverSocket = new ServerSocket(PORT); 
				serverSocket.setSoTimeout(2000);
				
				while (! terminate) {
					try {
						clientSocket = serverSocket.accept();
						new Thread(new ClientRunnable(clientSocket)).start(); 
					} catch (SocketTimeoutException te) {
					}
				}
				
			} catch (IOException e) {
				
			} finally {
				try {
					serverSocket.close();
				} catch (IOException e) { }
				Out.println("Server terminated");

			}
		}).start();
		
		// terminate on input from user
		Out.println("Terminate with ENTER");
		In.readLine(); 
		terminate = true; 
	}
		
	private class ClientRunnable implements Runnable {
		
		private final Socket clientSocket; 
		private BufferedReader reader; 
		private PrintWriter writer; 
		private String clientName; 

		public ClientRunnable(Socket clientSocket) {
			super();
			this.clientSocket = clientSocket;
		}


		@Override
		public void run() {
			try {
				this.reader = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
				this.writer = new PrintWriter(new OutputStreamWriter(clientSocket.getOutputStream()));
				// ---> HELO ipaddr
				writeLine(writer, String.format("%s:%s", HELO, clientSocket.getInetAddress().toString()));
				// <--- LOGIN name
				String login = readLine(reader); 
				String[] ls = login.split(":"); 
				clientName = ls[1]; 
				// ---> WHAT name
				writeLine(writer, String.format("%s:%s", WHAT, clientName )); 
				// while not END
				// <--- REQST req
				String req = readLine(reader); 	
				while (! req.startsWith(END.name())) {
					String[] reqs = req.split(":"); 
					// case of req 
					// ---> REPLY rep 
					writeLine(writer, String.format("%s:%s", REPLY, "You get " + reqs[1]));
					// <--- REQST req
					req = readLine(reader); 	
				}
				writeLine(writer, String.format("%s:%s", END, clientName));
				// <--- LOGOUT name
				String line = readLine(reader); 
				// ---> BUY name
				writeLine(writer, String.format("%s:%s", BYE, clientName));

			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				try {
					clientSocket.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}

			
		}
		
	}

}
